#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"

#include "S2ECore.h"
#include "socket.h"

#include "stm32f10x_dma.h"
#include "stm32f10x_tim.h"

#define S2ECoreDebug

QueueHandle_t xQueueS2E, xQueueE2S;
static uint8_t u8aS2EBuffer[QUEUE_BUFFER_RX_SIZE];
static uint8_t u8aE2SBuffer[QUEUE_BUFFER_TX_SIZE];
static uint32_t u32S2EBufLoc;
static uint32_t u32E2SBufLoc;
static uint32_t u32S2ECurrDMASize = ETH_MAX_BUF_SIZE;
static uint32_t u32E2SCurrDMASize = 2048;
static uint8_t u8vEthernetMode = ETH_MODE_TCPS;//todo: move to eeprom

void vS2EHardwareInit(void)
{
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);

	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2 , ENABLE);

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3 , ENABLE);

	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);

	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);

	GPIO_InitTypeDef GPIO_InitStructure;

	/* Configure UART2 Tx (PA3) as alternate function push-pull *///DMA1 ch7
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	/* Configure UART3 Rx (PB11) as input floating *///DMA1 ch3
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOB, &GPIO_InitStructure);


	NVIC_InitTypeDef NVIC_InitStructure;

	NVIC_InitStructure.NVIC_IRQChannel = TIM2_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	/*for uart recv interrupt*/
	NVIC_InitStructure.NVIC_IRQChannel = USART3_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	/*for S2E Buffer Full*/
	NVIC_InitStructure.NVIC_IRQChannel = DMA1_Channel3_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	/*for Send Complete*/
	NVIC_InitStructure.NVIC_IRQChannel = DMA1_Channel7_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;

	/* Compute the prescaler value */
	uint16_t PrescalerValue;
	PrescalerValue = (uint16_t) (SystemCoreClock / 24000000) - 1;

	/* Time base configuration */
	TIM_TimeBaseStructure.TIM_Prescaler = 7200;//1MHz -> 100us
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseStructure.TIM_Period = CMD_TIME_GAP;//0.1sec
	TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;
	TIM_TimeBaseStructure.TIM_RepetitionCounter = 0;
	TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);//todo
	TIM_ClearITPendingBit(TIM2,TIM_IT_Update);
	TIM_ITConfig(TIM2,TIM_IT_Update,ENABLE);

	USART_InitTypeDef USART_InitStructure;

	USART_InitStructure.USART_BaudRate = 115200;
	USART_InitStructure.USART_WordLength = USART_WordLength_8b;
	USART_InitStructure.USART_StopBits = USART_StopBits_1;
	USART_InitStructure.USART_Parity = USART_Parity_No;
	USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	USART_InitStructure.USART_Mode = USART_Mode_Rx;
	USART_Init(USART3, &USART_InitStructure);
	USART_ITConfig(USART3,USART_IT_RXNE,ENABLE);
	//USART_ClearFlag(UART3,USART_FLAG_TXE);
	//USART_ClearITPendingBit(UART3,USART_IT_TXE);

	USART_InitStructure.USART_Mode = USART_Mode_Tx;
	USART_Init(USART2, &USART_InitStructure);
	//USART_ClearFlag(UART2,USART_FLAG_TXE);
	//USART_ClearITPendingBit(UART2,USART_IT_TXE);

	DMA_InitTypeDef DMA_InitStructure;

	DMA_InitStructure.DMA_PeripheralBaseAddr = USART3_DR_Address;
	DMA_InitStructure.DMA_MemoryBaseAddr = u8aS2EBuffer;
	DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;
	DMA_InitStructure.DMA_BufferSize = ETH_MAX_BUF_SIZE;
	DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
	DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
	DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
	DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
	DMA_InitStructure.DMA_Priority = DMA_Priority_High;
	DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
	DMA_Init(DMA1_Channel3, &DMA_InitStructure);
	DMA_ITConfig(DMA1_Channel3,DMA_IT_TC,ENABLE);

	DMA_InitStructure.DMA_PeripheralBaseAddr = USART2_DR_Address;
	DMA_InitStructure.DMA_MemoryBaseAddr = u8aE2SBuffer;
	DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
	DMA_InitStructure.DMA_BufferSize = 0;
	DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
	DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
	DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
	DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
	DMA_InitStructure.DMA_Priority = DMA_Priority_High;
	DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
	DMA_Init(DMA1_Channel7, &DMA_InitStructure);
	DMA_ITConfig(DMA1_Channel7,DMA_IT_TC,ENABLE);

	//DMA_Cmd(DMA1_Channel7,ENABLE);
	DMA_Cmd(DMA1_Channel3,ENABLE);

	USART_DMACmd(USART3,USART_DMAReq_Rx,ENABLE);
	USART_Cmd(USART3, ENABLE);

	USART_DMACmd(USART2,USART_DMAReq_Tx,ENABLE);
	USART_Cmd(USART2, ENABLE);
	//Change DMA counter using DMA_SetCurrDataCounter(DMAx_channel,size );
	//Change DMA memory address using "DMAy_Channelx->CMAR = 0xXXXXXXXX" <--!!!
	xQueueS2E = xQueueCreate( QUEUE_MAX_SIZE, sizeof( mQueue ) );
	xQueueE2S = xQueueCreate( QUEUE_MAX_SIZE, sizeof( mQueue ) );
}

void vTskS2ECore( void *pbParameters )
{
	uint8_t  destip[4] = {222,98,173,244};
	uint16_t destport = 5001;
	uint8_t  tdestip[4] = {0,};
	uint16_t tdestport = 0;
	BaseType_t xReturn;
	TaskHandle_t xEthSendHandle, xEthRecvHandle;
	mQueue messageQueue;
	uint32_t melong,len,temp;
	//Queue create
	xQueueS2E = xQueueCreate( QUEUE_MAX_SIZE, sizeof( mQueue ) );
	xQueueE2S = xQueueCreate( QUEUE_MAX_SIZE, sizeof( mQueue ) );
	if(xQueueE2S == NULL || xQueueS2E == NULL)
	{
#ifdef S2ECoreDebug
		printf("Fail to Create queue.\r\n");
#endif
		while(1){}
	}
	//Ethernet task create
	xReturn = xTaskCreate(vTskEthRecv,"EthRecv",configMINIMAL_STACK_SIZE,NULL,tskIDLE_PRIORITY+1,&xEthRecvHandle);//??
	if(xReturn != pdPASS)
	{
#ifdef S2ECoreDebug
		printf("Fail to Create Ethernet Recv Task.\r\n");
#endif
		while(1){}
	}
	socket(DEFAULT_SOCKET_NO, Sn_MR_UDP, 5000, SF_IO_NONBLOCK);
	len = 2048;
	u32E2SBufLoc = 0;
	mQueue TestQueue;
	temp = 0;
	while(1)
	{
		if(xQueueReceive(xQueueS2E,&messageQueue,0) == pdTRUE)
		{
			sendto(DEFAULT_SOCKET_NO,messageQueue.start,messageQueue.size,destip,destport);
		}

		melong = recvfrom(DEFAULT_SOCKET_NO,u8aE2SBuffer+u32E2SBufLoc,len,tdestip,&tdestport);
		if(melong > 0)
		{
			TestQueue.size = melong;
			TestQueue.start = u8aE2SBuffer+u32E2SBufLoc;
			//if(uxQueueSpacesAvailable(xQueueE2S))
			//{
				xQueueSend(xQueueE2S,&TestQueue,0);
				u32E2SBufLoc += TestQueue.size;
				if(u32E2SBufLoc == QUEUE_BUFFER_TX_SIZE)
					u32E2SBufLoc = 0;
				if(u32E2SBufLoc + 2048 > QUEUE_BUFFER_TX_SIZE)
				{
					len = QUEUE_BUFFER_TX_SIZE - u32E2SBufLoc;
				}
			//}
		}
//		switch(getSn_SR(sn))
//		{
//			case SOCK_ESTABLISHED :
//				if(getSn_IR(sn) & Sn_IR_CON)
//				{
//					setSn_IR(sn,Sn_IR_CON);
//				}
//				if((size = getSn_RX_RSR(sn)) > 0) // Don't need to check SOCKERR_BUSY because it doesn't not occur.
//				{
//					if(size > DATA_BUF_SIZE) size = DATA_BUF_SIZE;
//						ret = recv(sn, buf, size);
//
//					if(ret <= 0) return ret;      // check SOCKERR_BUSY & SOCKERR_XXX. For showing the occurrence of SOCKERR_BUSY.
//						sentsize = 0;
//
//					while(size != sentsize)
//					{
//						ret = send(sn, buf+sentsize, size-sentsize);
//						if(ret < 0)
//						{
//							close(sn);
//							return ret;
//						}
//						sentsize += ret; // Don't care SOCKERR_BUSY, because it is zero.
//					}
//				}
//				break;
//			case SOCK_CLOSE_WAIT :
//				if((ret = disconnect(sn)) != SOCK_OK) return ret;
//				break;
//			case SOCK_INIT :
//				if( (ret = listen(sn)) != SOCK_OK) return ret;
//				break;
//			case SOCK_CLOSED:
//				if((ret = socket(sn, Sn_MR_TCP, port, 0x00)) != sn) return ret;
//				break;
//			default:
//				break;
//		}

		/*
		 * vTaskSuspend(xEthSendHandle);
		 * or
		 * vTaskSuspend(xEthRecvHandle);
		 *
		 * and
		 *
		 * vTaskResume(xEthSendHandle);
		 * or
		 * vTaskResume(xEthRecvHandle);
		 */
	}
}

void vTskEthSend( void *pvParameters )
{
}

void vTskEthRecv( void *pvParameters )
{
	mQueue messageQueue;
	uint32_t temp = 0;
	while(1)
	{
		if((DMA1_Channel7->CCR & DMA_CCR1_EN) == 0)
		{
			if(xQueueReceive(xQueueE2S,&messageQueue,0) == pdTRUE)
			{
				DMA1_Channel7->CMAR = messageQueue.start;
				DMA_SetCurrDataCounter(DMA1_Channel7, messageQueue.size);
				DMA_Cmd(DMA1_Channel7,ENABLE);
			}
		}
	}
}

void TIMER_IRQ_HANDLER(void)//Time out interrupt handler
{
	BaseType_t xHigherPriorityTaskWoken;
	mQueue TestQueue;
	UBaseType_t uxSavedInterruptStatus;
	uxSavedInterruptStatus = taskENTER_CRITICAL_FROM_ISR();
	TIM_Cmd(TIM2, DISABLE);
	//Input to queue
	TestQueue.size = u32S2ECurrDMASize-DMA_GetCurrDataCounter(DMA1_Channel3);
	TestQueue.start = u8aS2EBuffer+u32S2EBufLoc;
	xQueueSendFromISR(xQueueS2E,&TestQueue,&xHigherPriorityTaskWoken);
	u32S2EBufLoc += TestQueue.size;// ̼ Ʈ
	DMA_Cmd(DMA1_Channel3,DISABLE);
	//dma Ҵ  ũⰡ ť  ũ⺸ ũ    ŭ ҴѴ.
	if(u32S2EBufLoc+ETH_MAX_BUF_SIZE > QUEUE_BUFFER_RX_SIZE)
	{
		u32S2ECurrDMASize = QUEUE_BUFFER_RX_SIZE - u32S2EBufLoc;//   
		DMA1_Channel3->CMAR = u8aS2EBuffer + u32S2EBufLoc;
		DMA_SetCurrDataCounter(DMA1_Channel3, u32S2ECurrDMASize);//ǹ .
	}
	else
	{
		u32S2ECurrDMASize = ETH_MAX_BUF_SIZE;
		DMA1_Channel3->CMAR = u8aS2EBuffer + u32S2EBufLoc;
		DMA_SetCurrDataCounter(DMA1_Channel3, u32S2ECurrDMASize);
	}
	DMA_Cmd(DMA1_Channel3,ENABLE);
	//dma interrupt ʱȭ   . ׷ ڴ.
	TIM_ClearITPendingBit(TIM2,TIM_IT_Update);
	taskEXIT_CRITICAL_FROM_ISR( uxSavedInterruptStatus );
}

void DMARECV_IRQ_HANDLER(void)//DMA Buffer Full interrupt handler
{
	BaseType_t xHigherPriorityTaskWoken;
	mQueue TestQueue;
	UBaseType_t uxSavedInterruptStatus;
	uxSavedInterruptStatus = taskENTER_CRITICAL_FROM_ISR();
	TestQueue.size = u32S2ECurrDMASize-DMA_GetCurrDataCounter(DMA1_Channel3);
	TestQueue.start = u8aS2EBuffer+u32S2EBufLoc;
	xQueueSendFromISR(xQueueS2E,&TestQueue,&xHigherPriorityTaskWoken);
	u32S2EBufLoc += TestQueue.size;// ̼ Ʈ
	if(u32S2EBufLoc == QUEUE_BUFFER_RX_SIZE)
		u32S2EBufLoc = 0;
	DMA_Cmd(DMA1_Channel3,DISABLE);
	//dma Ҵ  ũⰡ ť  ũ⺸ ũ    ŭ ҴѴ.
	if(u32S2EBufLoc+ETH_MAX_BUF_SIZE > QUEUE_BUFFER_RX_SIZE)
	{
		u32S2ECurrDMASize = QUEUE_BUFFER_RX_SIZE - u32S2EBufLoc;//   
		DMA1_Channel3->CMAR = u8aS2EBuffer + u32S2EBufLoc + TestQueue.size;
		DMA_SetCurrDataCounter(DMA1_Channel3, u32S2ECurrDMASize);//ǹ .
	}
	else
	{
		u32S2ECurrDMASize = ETH_MAX_BUF_SIZE;
		DMA1_Channel3->CMAR = u8aS2EBuffer + u32S2EBufLoc;
		DMA_SetCurrDataCounter(DMA1_Channel3, u32S2ECurrDMASize);
	}
	//Timer interrupt ʱȭ   .
	DMA_ClearFlag(DMA1_FLAG_TC3);
	DMA_ClearITPendingBit(DMA1_IT_TC3);
	DMA_Cmd(DMA1_Channel3,ENABLE);
	taskEXIT_CRITICAL_FROM_ISR( uxSavedInterruptStatus );
}

void DMASEND_IRQ_HANDLER(void)//DMA Transmit done interrupt handler
{
	UBaseType_t uxSavedInterruptStatus;
	uxSavedInterruptStatus = taskENTER_CRITICAL_FROM_ISR();
	DMA_Cmd(DMA1_Channel7,DISABLE);
	//confirm that dma transmit done interrupt occur
	//stop dma
	//reset dma memory address(if queue is not empty)
	//start dma
	//clear dma interrupt flag
	DMA_ClearFlag(DMA1_FLAG_TC7);
	DMA_ClearITPendingBit(DMA1_IT_TC7);
	taskEXIT_CRITICAL_FROM_ISR( uxSavedInterruptStatus );
}

void SERIAL_IRQ_HANDLER(void)//Uart recv interrupt handler
{
	UBaseType_t uxSavedInterruptStatus;
	uxSavedInterruptStatus = taskENTER_CRITICAL_FROM_ISR();
	//confirm that uart recv interrupt occur
	//TIM_Cmd(TIM2,DISABLE);
	TIM2->CNT = 0;
	//TIM_Cmd(TIM2,ENABLE);
	TIM2->CR1 |= TIM_CR1_CEN;
	//USART_ClearFlag(USART3,USART_FLAG_RXNE|USART_IT_ORE);
	USART_ClearITPendingBit(USART3,USART_IT_RXNE|USART_IT_ORE);
	//TIM_ClearITPendingBit(TIM2,TIM_IT_Update);
	taskEXIT_CRITICAL_FROM_ISR( uxSavedInterruptStatus );
}
