#include "stm32f10x.h"

/* low level(Hardware) initialization */
#include "IoTEVB.h"
#include "FreeRTOSConfig.h"
#include "W5500HardwareDriver.h"
#include "S2ECore.h"

/* Wiznet BSD Socket library */
#include "wizchip_conf.h"
#include "socket.h"

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "timers.h"

#include <stdio.h>

/* Application library */
#include "httpServer.h"
#include "webpage.h"

#define TCP_PORT 	60000
#define UDP_PORT 	60001

#define TCP_SOCKET	0
#define UDP_SOCKET	1

#define MAX_HTTPSOCK	6
uint8_t socknumlist[] = {2, 3, 4, 5, 6, 7};

uint8_t RX_BUF[2048];
uint8_t TX_BUF[2048];

wiz_NetInfo gWIZNETINFO = { .mac = {0,},
							.ip = {222,98,173,249},
							.sn = {255, 255, 255, 192},
							.gw = {222, 98, 173, 254},
							.dns = {168, 126, 63, 1},
							.dhcp = NETINFO_STATIC};

void vLEDTask( void *pvParameters );
void vHTTPTask( void *pvParameters );
void vTimerCallBack( TimerHandle_t xTimer );

int main(void)
{
	USART1Initialze();
		printf("USART initialized.\n\r");
	I2C1Initialize();
		printf("I2C initialized.\n\r");
	MACEEP_Read(gWIZNETINFO.mac,0xfa,6);

	/*W5500 initialization.*/
	W5500HardwareInitilize();
		printf("W5500 hardware interface initialized.\n\r");

	W5500Initialze();
		printf("W5500 IC initialized.\n\r");

	led_initialize();
	/*Set network informations*/
	wizchip_setnetinfo(&gWIZNETINFO);

	print_network_information();

	vS2EHardwareInit();

	xTaskCreate(vLEDTask, "LEDTask", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY+1, NULL );
	xTaskCreate(vHTTPTask, "HTTPTask", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY+1, NULL );
	xTaskCreate(vTskS2ECore, "S2ECoreTask", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY+1, NULL );
	/*
	 * USER CODE HERE
	 *
	 */
	vTaskStartScheduler();
}

void vLEDTask( void *pvParameters )
{
	led_ctrl led1,led2;
	led_initialize();
	while(1)
	{
		led1Ctrl(ON);
		vTaskDelay(1000);
		led1Ctrl(OFF);
		vTaskDelay(1000);
	}
}

void vHTTPTask( void *pvParameters )
{
	int i;
	TimerHandle_t xTimer;
	xTimer = xTimerCreate("HttpTimer",1000,pdTRUE,0,vTimerCallBack);
	xTimerStart(xTimer,0);
	httpServer_init(TX_BUF, RX_BUF, MAX_HTTPSOCK, socknumlist);		// Tx/Rx buffers (1kB) / The number of W5500 chip H/W sockets in use
	reg_httpServer_cbfunc(NVIC_SystemReset, NULL); 					// Callback: NXP MCU Reset
	reg_httpServer_webContent((uint8_t *)"index.html", (uint8_t *)index_page);				// index.html 		: Main page example
	reg_httpServer_webContent((uint8_t *)"netinfo.html", (uint8_t *)netinfo_page);			// netinfo.html 	: Network information example page
	reg_httpServer_webContent((uint8_t *)"netinfo.js", (uint8_t *)wiz550web_netinfo_js);	// netinfo.js 		: JavaScript for Read Network configuration 	(+ ajax.js)
	reg_httpServer_webContent((uint8_t *)"img.html", (uint8_t *)img_page);					// img.html 		: Base64 Image data example page
	while(1)
	{
		for(i = 0; i < MAX_HTTPSOCK; i++)	httpServer_run(i);
	}
}

void vTimerCallBack( TimerHandle_t xTimer )
{
	httpServer_time_handler();
}
